#!/bin/bash

code_hello_server="HOWDY";
code_hello_client="HELLO";
code_declare_proto="PROTO";
code_declare_time="TIME";
code_declare_token="TOKEN";
code_commitment="COMMIT";
code_flip_result="FLIPPED";
code_reveal_bet="BET";
code_bye="BYEBYE";
code_error="ERROR";

random_device="/dev/urandom";

function proto_expect(){
	expected_code="$1";
	if [ -n "$2" ]; then
		error_message="${2}";
	else
		error_message="expected ${expected_code}";
	fi;

	read response;
	response_code="$(echo $response | awk '{ print $1; }')";
	response_data="$(echo $response | awk '{ $1=""; print $0; }')";

	if [ "${response_code}" != "${expected_code}" ]; then
		proto_error "${error_message}";
		return 1;
	else
		echo $response_data;
		return 0;
	fi;
}

function proto_error(){
	echo "error";
}

function get_token(){
	export own_private_token="$(head -c 8 "${random_device}" | hexdump -ve '1/1 "%.2x"')";
	export own_public_token="$(head -c 8 "${random_device}" | hexdump -ve '1/1 "%.2x"')";
}

function get_time(){
	export own_time="$(date +%s)";
	echo "${own_time}";
}

function declare_info(){
	echo "${code_declare_time} $(get_time)";
	get_token;
	echo "${code_declare_token} ${own_public_token}";
}

function get_declare(){
	export other_time="$(proto_expect "${code_declare_time}")";
	export other_public_token="$(proto_expect "${code_declare_token}")";
}

function get_session_token(){
	token_string="$(get_combined_token)";

	export session_token="$(hash_string "${token_string}")";
}

function code_bet(){
	bet="${1}";

	echo "${session_token}::${bet}::${own_private_token}";
}

function decode_bet(){
	code="${1}";

	coded_session="$(echo $code | awk -F'::' '{ print $1; }';)";
	if [ "${coded_session}" == "${session_token}" ]; then	
		echo $code | awk -F'::' '{ print $2; }';	
	else
		proto_error "Your commitment ${commitment} was not right, missing right session.";
		return 1;
	fi;
}

function show_information(){
	text="$1";
	title="$2";

	if [ -z "$title" ]; then
		title="remote-coin-flip";
	fi;

	if [ -n "$DISPLAY" ] && which zenity &>/dev/null; then
		zenity --info --text="${text}" --title="${title}";
	else
		echo -en "\n${text}\n" >&2;
		read <&2;
	fi;
}

function get_winner(){
	text="The client's bet was ${bet}, and the coin gave ${coin}.";

	if [ "${bet}" == "${coin}" ]; then
		text="${text}\n\nThe client wins this round!"
	else
		text="${text}\n\nThe server wins this round!"
	fi;

	show_information "${text}";
}

function hash_string(){
	echo "$*" | sha1sum | awk '{ print $1; }';
}

function get_port(){
	echo 54300;
}

function get_do_script(){
	script="$(readlink -f $0)";
	param="do";
	
	if [ -n "${DEBUG}" ]; then
		pipe="| tee $(tty)";
	else
		pipe="";
	fi;


	echo "${script} ${param} ${pipe}";
}

function goodbye(){
	echo "${code_bye} Bye bye.";
}

function ask_coin(){
	question="$1";
	title="$2";

	if [ -z "$title" ]; then
		title="remote-coin-flip";
	fi;

	if [ -n "$DISPLAY" ] && which zenity &>/dev/null; then
		zenity --question --text="${question}" --ok-label="Heads" --cancel-label="Tails" --title="${title}";
		if [ $? -eq 0 ]; then
			echo "heads";
		else
			echo "tails";
		fi;
	else
		while true; do
			echo -en "${question} (h/t): " >&2;
			read coin <&2;
    		case $coin in
				h|heads)
					echo "heads";
					break;;
				t|tails)
					echo "tails";
					break;;
			esac;
		done;
	fi;
}

