#!/bin/bash

source common.sh;

function identify(){
	echo "${code_hello_server} traveller.";
	export other_hello="$(proto_expect "${code_hello_client}" "Client should come say hello")";
	test "${other_hello}" != "error";
}

function get_combined_token(){
	echo "${own_token}::${other_token}";
}

function get_commitment(){
	export commitment="$(proto_expect "${code_commitment}" "Wait, are you going to commit or what?")";
	test "${commitment}" != "error";
}

function get_coin_flipped(){
	export coin="$(ask_coin "Flip a coin.\nWhat\'s the result?")";
}

function send_coin_result(){
	echo "${code_flip_result} ${coin}";
}

function get_bet(){
	coded_bet="$(proto_expect "${code_reveal_bet}" "Do you want me to guess your bet?")";
	test "${coded_bet}" != "error";
	export bet="$(verify_bet "${coded_bet}")";
}

function verify_bet(){
	coded_bet="${1}";
	hashed_bet="$(hash_string "${coded_bet}")";

	if [ "${hashed_bet}" == "${commitment}" ]; then
		decode_bet "${coded_bet}";
		return $?;
	else
		proto_error "Your commitment string was not right at all.";
		return 1;
	fi;
}

function start_server(){
	port="$(get_port)";
	echo "Starting server (listening on :${port})..." >&2;

	nc -l -p "${port}" -c "$(get_do_script)";
}

function do_server(){
	set -e;
	identify;
	declare_info;
	get_declare;
	get_session_token;
	get_commitment;
	get_coin_flipped;
	send_coin_result;
	get_bet;
	get_winner;
	goodbye;
}

if [ $# -ne 1 ] || [ "$1" != "do" ]; then
	start_server;
else
	do_server;
fi;
